'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
var _arguments = arguments;
var _ = require('lodash');
var mongo = require('mongodb');
var itsSet = require('its-set');
var async = require('async');
var MongoClient = mongo.MongoClient;
var ObjectID = mongo.ObjectID;

exports.default = function (_data) {
  var data = void 0;
  if (_.isString(_data)) {
    data = require(_data);
  } else {
    data = _data;
  }
  var objs = {};
  var refs = {};
  var options = void 0;
  var next = void 0;

  var args = Array.prototype.slice.call(_arguments);
  if (args.length === 2) {
    next = args[1];
    options = {};
  } else {
    next = args[2];
    options = args[1] || {};
  }

  var opts = _.merge({
    host: '127.0.0.1',
    port: 27017,
    db: 'mongo_fixture_test',
    drop: true
  }, options);

  var getRefs = function getRefs(d) {
    if (itsSet(d.__ref)) {
      refs[d.__collection] = refs[d.__collection] || {};
      refs[d.__collection][d.__ref] = d;
    }
    _.forIn(d, function (v) {
      return _.isObject(v) ? getRefs(v) : null;
    });
  };

  var analyseObj = function analyseObj(d) {
    var col = d.__collection;
    if (itsSet(col)) {
      objs[col] = objs[col] || [];
      objs[col].push(d);
    }

    _.forIn(d, function (v) {
      if (_.isObject(v)) {
        analyseObj(v);
      }
    });
  };

  var assignIds = function assignIds(d) {
    if (!itsSet(d.__duplicate) && itsSet(d.__collection)) {
      d._id = new ObjectID();
    }
    _.forIn(d, function (v, k) {
      if (_.isObject(v)) {
        if (itsSet(v.__id) && itsSet(v.__collection) && itsSet(v.__ref)) {
          d[k] = refs[v.__collection][v.__ref]._id;
        } else {
          assignIds(v);
        }
      }
    });
  };

  var flatten = function flatten(d) {
    _.forIn(d, function (v, k, o) {
      if (_.isObject(v)) {
        var dup = v.__duplicate;
        var col = v.__collection;
        if (itsSet(col)) {
          if (itsSet(dup)) {
            var dupObj = _.findWhere(objs[col], { __ref: dup });
            if (!itsSet(dupObj)) {
              next(new Error("#{col} with '__ref': #{dup} can't be found"));
            }
            o[k] = dupObj._id;
          } else {
            o[k] = v._id;
          }
        } else {
          flatten(v);
        }
      }
    });
  };

  var clean = function clean(arr) {
    var i = -1;
    var cleanNext = function cleanNext() {
      if (i++ < arr.length) {
        var d = arr[i];
        if (_.isObject(d)) {
          delete d.__ref;
          delete d.__collection;
          if (itsSet(d.__duplicate)) {
            arr.splice(i, 1);
            i--;
          }
        }
        cleanNext();
      }
    };
    cleanNext();
  };

  var drop = function drop(db, next2) {
    _.forIn(objs, function (v, k) {
      var collection = db.collection(k);
      collection.drop();
    });
    next2();
  };

  var populate = function populate(db, next2) {
    async.forEachOf(objs, function (v, k, next3) {
      var collection = db.collection(k);
      async.eachSeries(v, function (d, next4) {
        collection.insert(d, next4);
      }, next3);
    }, next2);
  };

  _.each(data, function (d) {
    return getRefs(d);
  });

  _.each(data, function (d) {
    return analyseObj(d);
  });

  _.forIn(objs, function (v) {
    return _.each(v, function (obj) {
      return assignIds(obj);
    });
  });

  _.forIn(objs, function (v) {
    return _.each(v, function (obj) {
      return flatten(obj);
    });
  });

  _.forIn(objs, function (v) {
    return clean(v);
  });

  MongoClient.connect('mongodb://#{opts.host}:#{opts.port}/#{opts.db}', function (err, db) {
    if (itsSet(err)) throw err;
    if (opts.drop) {
      drop(db, function () {
        populate(db, function () {
          db.close();
          next(null);
        });
      });
    } else {
      populate(db, function () {
        db.close();
        next(null);
      });
    }
  });
};